
# q-expansion of newform 75.10.a.b, downloaded from the LMFDB on 28 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 75
weight = 10
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-18], [-81], [0], [-9128], [21132], [-31214], [279342], [144020], [1763496], [4692510], [-369088], [-9347078], [-7226838], [23147476], [-22971888], [-78477174], [-20310660], [-179339938], [-274528388], [-36342648], [247089526], [191874800], [276159276], [-678997350], [567657502], [1622823462], [1751025976], [1542957012], [457665230], [-3267943794], [-928879328], [-988658748], [57325302], [-4650524980], [-1402362090], [10154751032], [-9366053438], [734779996], [-1555841448], [-19044819054], [4328516340], [-956757418], [-17643820608], [-13099326914], [-4745885958], [-19210229080], [-18127312108], [-3910810304], [35812588572], [5304447110], [25417154646], [-63349422480], [101928795122], [96474476412], [7995908862], [-106269250824], [-100886359410], [-28634476048], [-41225078198], [121800028122], [-23282019164], [91280097306], [43906990732], [-76483478088], [-268488258554], [-152055600318], [-263057340148], [-297182797778], [-140218516188], [-314890729090], [531194758686], [-497683661400], [-277743102608], [-255318851414], [81210280700], [413122928256], [253052022150], [65356172242], [-240886414638], [-378340063270], [968765698980], [-143788604218], [206889561072], [48188303566], [437941020440], [-758664977724], [-137279666430], [-131091223178], [-803507833458], [1785816198736], [-644366062548], [-1877005156320], [-1493552300168], [-747313211988], [1012997786420], [-203398574904], [593698558590], [-1749501180918], [-320554846604], [350228556542], [2460370242652], [-1043558222958], [-3818493333684], [484062265530], [17704421372], [746985322942], [4339477542132], [-1901196887634], [-5496489625320], [4776982081562], [-824999503808], [1469852997946], [-5173730633898], [-4802740911700], [5939922195992], [6477383884482], [1745192195836], [-4972442023368], [1450572111666], [-3051747968940], [-1883147739658], [-7843585659554], [-2324197335078], [6892645350996], [11143009010612], [-5780148784578], [-3399020676250], [271271771280], [2138634975112], [-4600332337214], [8602445327780], [-1314074790504], [-7958097175408], [6959363757802], [-13863177783558], [13560448958210], [-4967402852934], [5258091267052], [-16181648783838], [-13927645283670], [13818981957932], [3485780849142], [12147214578856], [17070225294852], [5580709360670], [12864247581960], [-10346474541494], [-5632158492378], [-22828883213860], [-4629182781024], [-5336110351118], [7878899572722], [-13897291208564], [5866069308552], [-8522502315788], [16336850963472], [32471231370680], [-12457146318750], [-9290118716138], [-32483106483858], [41625787937292], [-18647906982714], [23298953952472], [-36362793092148], [-51814128153618], [25486233910056], [39283044182912], [-49838869596998], [36686271871730], [14553563227146], [-24095882221380], [-3310605163618], [-32350401835128], [27186531978166], [5915165795120], [16475176172250], [-25390838038948], [-39547499845338], [10270200695416], [-39881158647730], [25694365940512], [37425180751812], [5630823641626], [56408115279222], [-77598676831824], [13094776597590], [97304048686402], [56240305343836], [57649471455530], [103839985037952], [-65415572203394], [-26047543872204], [-2157004749868], [-59748591315618], [-75442469257188], [117283744442646], [-88973729776078], [-143693539080734], [59200404843582], [49902379089336], [119013362801550], [52607347685552], [-148922012005238], [-97365034370590], [52871998983660], [-154917260151678], [84812375704160], [-12646938358404], [28819830680010], [-161365810533748], [175575500621422], [-15935944437738], [-92456670673784], [-49367397780828], [-74025910937880], [42337416349802], [-195038025357008], [-1612098948078], [45039644638632], [39725570568546], [117731979648902], [-86886602319400], [66537702645570], [98407353018256], [184168284603372], [57664969777910], [260524529744166], [-164156171819040], [-38444771314568], [-136993664975508], [195766220605426], [50511446710580], [211170601564832], [183106076118282], [154442519144116], [-130417202600208], [-318124272874030], [-276853562871894], [41256279277020], [-348470657390808], [-293272961494644], [-39077501035588], [218993286705496], [402762872594990], [-320308679661714], [188319927066840], [359056606950592], [-142975950652188], [-108524295795220], [258456572160336], [108004026080482], [200487086180802], [-248958741220488], [493452009573770], [-61009308207054], [517416687243540], [148253780888822], [-125004197969948], [282839062517082], [-33392026055738], [66999731112736], [105100826564412], [-264761562296410], [-161075820709694], [523726292902302], [150078790167140], [208467266946030], [211308162263802], [490281806356036], [156707141957946], [417587128918382], [647958237593452], [204181955336806], [-246100255815040], [54504631307662], [-573629976011864], [691037046779652], [-815415097991650], [218595549984842], [-76787590772268], [363793381239456], [-572470982549368], [208652841374472], [393857688461222], [-226261145128268], [-448002277681968], [-230074688369234], [-388536587583318], [651991876115960], [124689116743650], [639330760439982], [-840289263188148], [530225761811526], [-733552036507188], [299084561251666], [-58877557193250], [-337260300840448], [400929026310666], [-1144969774880580], [-960154794810308], [1276722015036406], [-649420405832718], [-572410187011600], [-736830138287124], [-710183237420068], [300653057656222], [-880869509568108], [-860003905905970], [-415103010951240], [-1366611416122214], [-1361544882537744], [-1212696606430890], [268622509717602], [-1122773870569124], [1068076510978392], [1266147953043050], [-647395355251980], [-1230300540235008], [-1209729458405954], [491036927406930], [761674163213972], [-982694424513818], [925999560462942], [1242403668324736], [878034804093846], [317538915657842], [-1702588471940860], [1123000948939876], [1386117240197472], [452605323903066], [652720561824782], [-227605688606718], [2109282361531040], [-553040103171204], [825329642624012], [-1506068542854108], [-421324216504450], [-1017667019155554], [198662058279722], [274817795453872], [-2101892154206294], [230994340000902], [293632485392070], [-1648759219243288], [670354184055906], [-1706431859715420], [-1978739214193978], [-2217522808415468], [-1952335722006288], [443843830345482], [-997068161139388], [-767161296160778], [-1224802819113138], [-1781317378157744], [-202140098295370], [1049230282533606], [-601526336355360], [-1311058105160148], [-3069173484525618], [69288916872456], [-1002587419725158], [2305986087263562], [-39443535074448], [-1046695013054820], [1009955334943132], [2436993203971606], [-2900139340113198], [609379451817496], [-1261668536444518], [1405327981012452], [-215695510897300], [-1485501477648624], [594973577350710], [954622326501272], [-1442040597435998], [3452916783736980], [2765393788046112], [1067125365167326], [-1112136232429710], [1410362549658502], [3312452776064382], [3686855909743926], [-2495258170195448], [1141159566890142], [-411841420173340], [4186510941125016], [4389265662347792], [479910522681322], [-3911350699462844], [2084440829712192], [-2424005541896830], [667294281595386], [-819689979600180], [2841176033897452], [-1157620161057768], [-1399340131935194], [-814846619461120], [4995206924677290], [-2828817207462868], [3115993608073782], [-1788391621645090], [610320300272766], [-3195165292939568], [-2008556740309338], [-5694698911610970], [3502145893787912], [-3905143520933198], [-2627112242455758], [4824989377181836], [5270699630281860], [-4845344697946514], [4072100074011882], [-3482325932909724], [203686060572452], [2290083187889302], [2617134483810222], [-2824503822498240], [2156059825199512], [-4611290042736018], [-5950540194205464], [-4363031649728610], [-4357363715223878], [-2320124075240688], [546982024825980], [7725882224430646], [7340334693322482], [1088437896903276], [3073182879035610], [5670440025405212], [5669542189783800]]
